/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

#include "bbox.h"

// BBox Method Definitions
//! ajoute un point a l'aabox et renvoie la nouvelle aabox.
BBox Union( const BBox &b, const Vector &p )
{
    BBox ret = b;
    ret.pMin[0] = std::min(b.pMin[0], p[0]);
    ret.pMin[1] = std::min(b.pMin[1], p[1]);
    ret.pMin[2] = std::min(b.pMin[2], p[2]);
    ret.pMax[0] = std::max(b.pMax[0], p[0]);
    ret.pMax[1] = std::max(b.pMax[1], p[1]);
    ret.pMax[2] = std::max(b.pMax[2], p[2]);
    
    return ret;
}

//! ajoute une aabox dans une autre et renvoie la nouvelle aabox.
BBox Union( const BBox &b, const BBox &b2 )
{
    BBox ret;
    ret.pMin[0] = std::min(b.pMin[0], b2.pMin[0]);
    ret.pMin[1] = std::min(b.pMin[1], b2.pMin[1]);
    ret.pMin[2] = std::min(b.pMin[2], b2.pMin[2]);
    ret.pMax[0] = std::max(b.pMax[0], b2.pMax[0]);
    ret.pMax[1] = std::max(b.pMax[1], b2.pMax[1]);
    ret.pMax[2] = std::max(b.pMax[2], b2.pMax[2]);
    
    return ret;
}

BBox Intersection( const BBox& a, const BBox& b )
{
    BBox ret;
    ret.pMin[0]= std::max(a.pMin[0], b.pMin[0]);
    ret.pMin[1]= std::max(a.pMin[1], b.pMin[1]);
    ret.pMin[2]= std::max(a.pMin[2], b.pMin[2]);
    ret.pMax[0]= std::min(a.pMax[0], b.pMax[0]);
    ret.pMax[1]= std::min(a.pMax[1], b.pMax[1]);
    ret.pMax[2]= std::min(a.pMax[2], b.pMax[2]);        
    
    return ret;
}

void BBox::BoundingSphere( Vector& c, float& r ) const
{
    c= .5f * (pMin + pMax);
    r= Inside(c) ? Norm(c- pMax) : 0.f;
}
