/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
// Blob
// Changelog 05.01.24

// Blob include
#include "blobbox.h"

/*!
  \brief Create a point skeletal element.
  \param c Center.
  \param blend Blending function.
*/
BlobBox::BlobBox(const Vector& a, const Vector& b,const double& r,const double& s, const Vector& color):a(a), b(b), blend(Blend(r,s)){ 
  BlobBox::color=color;
  BlobBox::box = Box(std::min(a,b) - Vector(r, r, r), std::max(a,b) + Vector(r, r, r));
}

/*!
  \brief Overloaded to minimize function calls.
*/
double BlobBox::Intensity(const Vector& p)
{  
  // Box 
  if (!box.Inside(p)) return 0.0;
  double result=0.0;
  for (int i=0;i<3;i++)
      if (p[i]<a[i])
	result+=(p[i]-a[i])*(p[i]-a[i]);
      else if (p[i]>b[i])
	result+=(p[i]-b[i])*(p[i]-b[i]);

  return blend(result);	
}

Vector BlobBox::Color(const Vector& p){
  return color;
}
