/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
// Blob
// Changelog 05.01.24

// Blob include
#include "blobcone.h"

/*!
  \brief Create a point skeletal element.
  \param c Center.
  \param blend Blending function.
*/
BlobCone::BlobCone(const Vector& a, const Vector& b,const double& rayon1,const double& rayon2,const double& r,const double& s,const Vector& color):a(Vector(0,0,0)), b(b-a), v(a), rayon1(rayon1), rayon2(rayon2), blend(Blend(r,s)){ 
  double tmp = r+1.0;
  BlobCone::color=color;
  BlobCone::box = Box( Vector( a - Vector( rayon1*tmp, rayon1*tmp, rayon1*tmp) ), Vector( b + Vector( rayon1*tmp, rayon1*tmp, rayon1*tmp ) ) );
 
}

/*!
  \brief Overloaded to minimize function calls.
*/
double BlobCone::Intensity(const Vector& p)
{  
  Vector point=p-v;
  // Box 
  if (!box.Inside(point)) return 0.0;

  //http://liris.cnrs.fr/Documents/Liris-1297.pdf
  double result=0.0;
  double h = Norm(b - a) ;

  double S=rayon1-rayon2;
  double ss=sqrt(h*h + S*S);

  Vector u = Normalized(a-b);
  double x=(a-point)*u;
  double n2 = (a-point)*(a-point);
  double y2 = n2-x*x;
  double y=sqrt(y2);

  // Vector i(0,S/ss,h/ss);
  //Vector j(h/ss,-S/ss,0);
  
  if( x < 0 ){
    if( y2 < rayon1*rayon1 )
      result = x*x;
    else 
      result=(y-rayon1)*(y-rayon1)+x*x;
  }else if(y2<rayon2*rayon2){
    if(x>h)
      result = ( x - h ) * ( x - h );
    else result=0;
  }else{

    Vector normalPlan1 = u;
    CMatrix plan1;
    plan1(0,0) = 1.0f - normalPlan1[0]*normalPlan1[0];
    plan1(0,0) = 1.0f - normalPlan1[0]*normalPlan1[0];
    plan1(0,1) = -normalPlan1[0] * normalPlan1[1];
    plan1(0,2) = -normalPlan1[0] * normalPlan1[2];
    plan1(1,0) = -normalPlan1[1] * normalPlan1[0];
    plan1(1,1) = 1.0f - normalPlan1[1] * normalPlan1[1];
    plan1(1,2) = -normalPlan1[1] * normalPlan1[2];
    plan1(2,0) = -normalPlan1[2] * normalPlan1[0];
    plan1(2,1) = -normalPlan1[2] * normalPlan1[1];
    plan1(2,2) = 1.0f - normalPlan1[2] * normalPlan1[2];
    Vector pp1 = Normalized(plan1 * point);
    Vector ap =  pp1*rayon1+a;

    Vector normalPlan2 = u;
    CMatrix plan2;
    plan2(0,0) = 1.0f - normalPlan2[0]*normalPlan2[0];
    plan2(0,0) = 1.0f - normalPlan2[0]*normalPlan2[0];
    plan2(0,1) = -normalPlan2[0] * normalPlan2[1];
    plan2(0,2) = -normalPlan2[0] * normalPlan2[2];
    plan2(1,0) = -normalPlan2[1] * normalPlan2[0];
    plan2(1,1) = 1.0f - normalPlan2[1] * normalPlan2[1];
    plan2(1,2) = -normalPlan2[1] * normalPlan2[2];
    plan2(2,0) = -normalPlan2[2] * normalPlan2[0];
    plan2(2,1) = -normalPlan2[2] * normalPlan2[1];
    plan2(2,2) = 1.0f - normalPlan2[2] * normalPlan2[2];
    Vector pp2 = Normalized(plan2 * point);
    Vector bp =  pp2*rayon2+b;
   
    Vector up = Normalized(ap-bp);
    //Vector up = i;
    double xp=(ap-point)*up;
    double np2 = (point-ap)*(point-ap);
    double yp2 = np2-(xp*xp);
    //double yp=sqrt(yp2);
    if(xp<0)
      result = (y-rayon1)*(y-rayon1)+x*x;
    else if(xp > ss)
      result = yp2 + (xp - ss)*(xp - ss);
    else
      result =  yp2;
  }

  
  return blend(result);	
}

Vector BlobCone::Color(const Vector& p){
  return color;
}
