/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
// Blob
// Changelog 05.01.24

// Blob include
#include "blobcylinder.h"

/*!
  \brief Create a point skeletal element.
  \param c Center.
  \param blend Blending function.
*/
BlobCylinder::BlobCylinder(const Vector& a, const Vector& b,const double& rayon,const double& r,const double& s,const Vector& color):a(a), b(b), rayon(rayon), blend(Blend(r,s)){ 
  double tmp = r+1.0;
  BlobCylinder::color=color;
  BlobCylinder::box = Box( Vector( a - Vector( rayon*tmp, rayon*tmp, rayon*tmp) ), Vector( b + Vector( rayon*tmp, rayon*tmp, rayon*tmp ) ) );
}

/*!
  \brief Overloaded to minimize function calls.
*/
double BlobCylinder::Intensity(const Vector& p)
{  
  // Box 
  if (!box.Inside(p)) return 0.0;
  Vector v = box.Center() - p;
  double result=0.0;
  double hauteur = Norm(b - a) / 2.0;
  double distancePAxe= fabs(v*Normalized(a-b));
  double distancePOrtho = sqrt(v*v - distancePAxe*distancePAxe);

  // cf : http://liris.cnrs.fr/Documents/Liris-1297.pdf
  if( distancePAxe <= hauteur ) //inside
    if( distancePOrtho  < rayon ) result = 0.0;
    else result = ( distancePOrtho - rayon ) * ( distancePOrtho - rayon ); 		  
  else //out
    if ( distancePOrtho < rayon ) result = ( distancePAxe - hauteur ) * ( distancePAxe - hauteur );
    else
      result = ( distancePOrtho - rayon ) * ( distancePOrtho - rayon )  +  ( distancePAxe -  hauteur )*( distancePAxe -  hauteur );		
  
  return blend(result);	
}


Vector BlobCylinder::Color(const Vector& p){
  return color;
}
