/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
#include "blobobject.h"


blobObject::blobObject(const Vector& p,const CMatrix& r, const Vector& c ):position(p),rotation(r),color(c){

  av = new Vector[10000000];
  an = new Vector[10000000];
  ac = new Vector[10000000];
  at = new int[300000];

}

blobObject::blobObject(const blobObject& o):av(o.av),an(o.an),ac(o.ac),at(o.at),avn(o.avn),atn(o.atn),cutpos(o.cutpos),position(o.position),rotation(o.rotation),color(o.color),blob(o.blob){

}

blobObject::~blobObject(){
  delete [] av;
  delete [] an;
  delete [] ac;
  delete [] at;
  delete blob;
}

void blobObject::init(){

  blob=new Blob(
		new BlobVertex(Vector(0.0,0.0,0.0),1.0,3.0)
		,0.5);

  Box box=blob->GetBox().Cubic();
  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);
}

GLuint blobObject::GenerateBlob(){

  for (int i=0;i<avn;i++){
    an[i]=-blob->Gradient(av[i]);
    an[i]/=Norm(an[i]);
    av[i]=rotation*av[i];
    av[i]+=position;

  }



  GLuint list=glGenLists(1);
  // Optimize list
  glNewList(list,GL_COMPILE);

  GlutShade(color[0],color[1],color[2]);

  glBegin(GL_TRIANGLES);
  for (int i=0; i < atn; i++)
    {
      GlutShade(ac[at[i*3+0]][0],ac[at[i*3+0]][1],ac[at[i*3+0]][2]);
      Normal3D(an[at[i*3+0]]);
      Vertex3D(av[at[i*3+0]]);
      GlutShade(ac[at[i*3+1]][0],ac[at[i*3+1]][1],ac[at[i*3+1]][2]);
      Normal3D(an[at[i*3+1]]);
      Vertex3D(av[at[i*3+1]]);
      GlutShade(ac[at[i*3+2]][0],ac[at[i*3+2]][1],ac[at[i*3+2]][2]);
      Normal3D(an[at[i*3+2]]);
      Vertex3D(av[at[i*3+2]]);
    }
  glEnd();

  // Fin de la liste
  glEndList();
  return list;

}
GLuint blobObject::GenerateCut(){


  GLuint list=glGenLists(1);

  // Optimize list
  glNewList(list,GL_COMPILE);

  Quad quad(cutpos,Vector(0.0,0.0,1.0),101,8.0,blob);
  quad.Render();

  // Fin de la liste
  glEndList();

  return list;

}

void blobObjectTest1::init(){

  // BlobNode* b1 = new BlobCylinder(Vector(0,0,-1.0), Vector(0,0.0,1), 2, 1.0, 1.0);
  BlobNode* b1 = new BlobCone(Vector(0,0,0), Vector(0,0,8), 3,0, 1.0, 1.0);
  /*
    blob = new Blob(b1,0.5);
    Box box=blob->GetBox().Cubic();
    BlobNode* b2 = blob->Erode(box,100,1e-4, 0.1, 0.7, 0.2,0.5,Vector(0.,1,0.));

    blob=new Blob(
    new BlobSub(b1,b2)
    ,0.5);*/
  blob = new Blob(b1,0.5);
  Box box=blob->GetBox().Cubic();
  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);
}



void blobObjectTest2::init(){

  BlobNode* b1 =
    new BlobXor(
		new BlobCylinder(Vector(0,0,-1.0), Vector(0,0.0,1), 2, 1.0, 1.0,Vector(0,0,1)),
		new BlobCylinder(Vector(0,0,-2.0), Vector(0,0.0,2), 1, 1.0, 1.0,Vector(0,1,0)));
  /*        new BlobXor(
	    new BlobTore(Vector(0.0,1.0,0.0),Vector(0.0,1.0,0.0),4.0,3.0,1.0),
	    new BlobTore(Vector(0.0,1.0,1.0),Vector(0.0,0.0,1.0),4.0,3.0,1.0)
	    );          */
  //    Blob b1Tmp(b1,0.5);
  blob = new Blob(b1,0.5);
  Box box=blob->GetBox().Cubic();
  BlobNode* b2 = blob->Erode(box,100,1e-4,Vector(1,0,0), 0.1, 0.7, 0.2,0.5,Vector(0.,1,0.));
  /*
    BlobNode* b2=new BlobBox(Vector(-3.0,-4.0,-4.0),Vector(3.0,4.0,4.0),1.0,1.0);
    Blob b2Tmp(b2,0.5);
    Box box2Tmp=b2Tmp.GetBox().Cubic();
  */
  /*
    BlobNode* b3 = b1Tmp.Erode(box1Tmp,10,1e-4, 0.1, 0.7, 0.2,0.5,Vector(0.,1,0.));
    blob=new Blob(
    new BlobSub(b1,b3)
    ,0.5);*/
  /*  BlobNode* b2 = new BlobTore(Vector(0.0,1.0,0.0),Vector(0.0,1.0,0.0),4.0,3.0,1.0);
      Blob bTmp(b2,0.5);
      Box boxTmp=bTmp.GetBox().Cubic();
      BlobNode* b3 = bTmp.Erode(boxTmp,1000,1e-4, 0.1, 0.7, 0.05,0.2,Vector(0.,1,0.));

  */
  //    blob = new Blob(b1,0.5);


  //Box box=blob->GetBox().Cubic();
  blob=new Blob(
		new BlobSub(b1,b2)
		,0.5);

  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);
}



void blobMaison::init(){

  Vector toit1;
  Vector toit2;
  Vector mur;
  Vector chemine;

   if(irand(2,0)){
     toit1=Vector(1,0,0);
     toit2=Vector(0.9,0.9,0.9);
     mur=Vector(1,1,0.38);
     chemine=Vector(0.8,0.32,0.23);
  }else{
    toit1=Vector(0.1,0.2,0.7);
    toit2=Vector(0.9,0.1,0.1);
    mur=Vector(0.75,0.36,0.78);
    chemine=Vector(0.32,0.32,0.23);
    }

  BlobNode* b1 =
    new BlobBlend(
		  new BlobBlend(

				new BlobUnion(
					      new BlobCone(Vector(0,0,2.0), Vector(0,0.0,6), 4,0, 2.0, 1.0,toit1),
					      new BlobVertex(Vector(0,0,5.0), 4.0, 1.0,toit1)
					      ),
				new BlobUnion(
					      new BlobTore(Vector(0,0,1.8),Vector(0,0.1,1),4.0,1.0,1.0,toit1),
					      new BlobTore(Vector(0,0,1.8),Vector(0,-0.1,1),4.0,1.0,1.0,toit1)
					      )

				),
		  new BlobBlend(
				new BlobSub(
					    new BlobSub(
							new BlobVertex(Vector(0,0,-1.0), 5.0, 2.0,mur),
							//							new BlobIntersect(
							new BlobBox(Vector(-0.6,2.5,-3.0),Vector(0.6,4.0,0),0.4,1.0,Vector(0.38,0.23,0))
							),
					    new BlobBox(Vector(2.5,-0.6,-1.0),Vector(4,0.6,0),0.4,1.0,Vector(0,0,0.26))

					    ),
				new BlobBlend(
					      new BlobBox(Vector(-0.3,-3,-3.0),Vector(0.3,-2.4,5),1.0,1.0,chemine),
					      new BlobIntersect(
								new BlobBox(Vector(-0.3,-3,5),Vector(0.3,-2.4,10),1.0,1.0,chemine),
								new BlobBox(Vector(-0.1,-2.8,5),Vector(0.1,-2.6,7),1.0,1.0,chemine)
								)
					      )
				)
		  );

  BlobNode* bErode = new BlobUnion(
				   new BlobCone(Vector(0,0,2.0), Vector(0,0.0,6), 4,0, 2.0, 1.0,Vector(0.9,0.9,0.9)),
				   new BlobVertex(Vector(0,0,5.0), 4.0, 1.0,Vector(1,0.9,0.9))
				   );
  Blob bTmp(bErode,0.5);
  Box boxTmp=bTmp.GetBox().Cubic();
  BlobNode* b2 = bTmp.Erode(boxTmp,ERODE_V,1e-4,toit2, 0.1, 1., 0.05,0.2,Vector(0.0,0,1.0));
  blob=new Blob(
		new BlobSub( b1, b2)
		,0.5);
  Box box=blob->GetBox().Cubic();
  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);

}



void blobMaisonGrande::init(){

  Vector toit1(1,0,0);
  Vector toit2(0.9,0.9,0.9);
  Vector mur(1,1,0.38);
  Vector chemine(0.8,0.32,0.23);

  BlobNode* b1 =
    new BlobBlend(
		  new BlobBlend(

				new BlobUnion(
					      new BlobCone(Vector(0,0,2.0), Vector(0,0.0,6), 4,0, 2.0, 1.0,toit1),
					      new BlobVertex(Vector(0,0,5.0), 4.0, 1.0,toit1)
					      ),
				new BlobUnion(
					      new BlobTore(Vector(0,0,1.8),Vector(0,0.1,1),4.0,1.0,1.0,toit1),
					      new BlobTore(Vector(0,0,1.8),Vector(0,-0.1,1),4.0,1.0,1.0,toit1)
					      )

				),
		  new BlobBlend(
				new BlobSub(
					    new BlobSub(
							new BlobVertex(Vector(0,0,-1.0), 5.0, 2.0,mur),
							//							new BlobIntersect(
							new BlobBox(Vector(-0.6,2.5,-3.0),Vector(0.6,4.0,0),0.4,1.0,Vector(0.38,0.23,0))
							),
					    new BlobBox(Vector(2.5,-0.6,-1.0),Vector(4,0.6,0),0.4,1.0,Vector(0,0,0.26))

					    ),
				new BlobBlend(
					      new BlobBox(Vector(-0.3,-3,-3.0),Vector(0.3,-2.4,5),1.0,1.0,chemine),
					      new BlobIntersect(
								new BlobBox(Vector(-0.3,-3,5),Vector(0.3,-2.4,10),1.0,1.0,chemine),
								new BlobBox(Vector(-0.1,-2.8,5),Vector(0.1,-2.6,7),1.0,1.0,chemine)
								)
					      )
				)
		  );

  BlobNode* bErode = new BlobUnion(
				   new BlobCone(Vector(0,0,2.0), Vector(0,0.0,6), 4,0, 2.0, 1.0,Vector(0.9,0.9,0.9)),
				   new BlobVertex(Vector(0,0,5.0), 4.0, 1.0,Vector(1,0.9,0.9))
				   );
  Blob bTmp(bErode,0.5);
  Box boxTmp=bTmp.GetBox().Cubic();
  BlobNode* b2 = bTmp.Erode(boxTmp,100,1e-4,Vector(1,0.9,0.9), 0.1, 1., 0.05,0.2,Vector(0.0,0,1.0));
  blob=new Blob(
		new BlobSub( b1, b2)
		,0.5);
  Box box=blob->GetBox().Cubic();
  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);

}


void blobSchtroumpf::init(){
  Vector b(0.9,0.9,0.9);
  Vector c(0.1,0.1,0.8);
  BlobNode* b1 = new BlobUnion(
			       new BlobUnion(

					     new BlobBlend(
							   new BlobBlend(
									 new BlobBlend(
										       new BlobBlend(
												     new BlobVertex(Vector(0,0,0), 2.0, 1.0,c),//tête
												     new BlobTore(Vector(.8,0,0.2),Vector(1.0,0.2,0.10),.3,.2,1.0,c)
												     ),
										       new BlobTore(Vector(-.8,0,0.2),Vector(-1.0,0.2,0.10),.3,.2,1.0,c)
										       ),
									 new BlobVertex(Vector(0,1.0,0), 0.4, 1.0,c)//nez
									 ),

							   new BlobBlend(
									 new BlobVertex(Vector(0,0,-1.6), 1, 1.0,c),//cou
									 new BlobBlend(
										       new BlobBlend(
												     new BlobBlend(
														   new BlobVertex(Vector(0.1,0.0,-2), 1.2, 1.0,c),
														   new BlobVertex(Vector(-0.1,0.0,-2), 1.2, 1.0,c)
														   ),
												     new BlobVertex(Vector(0.0,0.2,-2), 1.2, 1.0,c)
												     ),	//corps
										       new BlobBlend(
												     new BlobBlend(
														   new BlobVertex(Vector(0.5,0,-1.6), 1, 1.0,c),
														   new BlobBlend(
																 new BlobVertex(Vector(0.8,0,-1.5), 0.5, 1.0,c),
																 new BlobBlend(
																	       new BlobVertex(Vector(1,0,-1.5), 0.4, 1.0,c),
																	       new BlobBlend(
																			     new BlobVertex(Vector(1.2,0,-1.5), 0.4, 1.0,c),
																			     new BlobBlend(
																					   new BlobVertex(Vector(1.4,0,-1.5), 0.4, 1.0,c),
																					   new BlobBlend(
																							 new BlobVertex(Vector(1.7,0,-1.5), 0.4, 1.0,c),
																							 new BlobUnion(
																								       new BlobUnion(
																										     new BlobUnion(
																												   new BlobVertex(Vector(1.8,0,-1.2), 0.2, 1.0,c),
																												   new BlobVertex(Vector(1.9,0,-1.7), 0.2, 1.0,c)
																												   ),
																										     new BlobBlend(
																												   new BlobVertex(Vector(1.9,0,-1.6), 0.2, 1.0,c),
																												   new BlobVertex(Vector(2.0,0,-1.6), 0.2, 1.0,c)
																												   )
																										     ),
																								       new BlobBlend(
																										     new BlobVertex(Vector(1.9,0,-1.4), 0.2, 1.0,c),
																										     new BlobVertex(Vector(2.0,0,-1.4), 0.2, 1.0,c)
																										     )
																								       )
																							 )
																					   )
																			     )
																	       )
																 )
														   ),//bras 1
												     new BlobBlend(
														   new BlobVertex(Vector(-0.5,0,-1.6), 1, 1.0,c),
														   new BlobBlend(
																 new BlobVertex(Vector(-0.8,0,-1.5), 0.5, 1.0,c),
																 new BlobBlend(
																	       new BlobVertex(Vector(-1,0,-1.5), 0.4, 1.0,c),
																	       new BlobBlend(
																			     new BlobVertex(Vector(-1.2,0,-1.5), 0.4, 1.0,c),
 new BlobBlend(
																					   new BlobVertex(Vector(-1.4,0,-1.5), 0.4, 1.0,c),
																					   new BlobBlend(
																							 new BlobVertex(Vector(-1.7,0,-1.5), 0.4, 1.0,c),
																							 new BlobUnion(
																								       new BlobUnion(
																										     new BlobUnion(
																												   new BlobVertex(Vector(-1.8,0,-1.2), 0.2, 1.0,c),
																												   new BlobVertex(Vector(-1.9,0,-1.7), 0.2, 1.0,c)
																												   ),
																										     new BlobBlend(
																												   new BlobVertex(Vector(-1.9,0,-1.6), 0.2, 1.0,c),
																												   new BlobVertex(Vector(-2.0,0,-1.6), 0.2, 1.0,c)
																												   )
																										     ),
																								       new BlobBlend(
																										     new BlobVertex(Vector(-1.9,0,-1.4), 0.2, 1.0,c),
																										     new BlobVertex(Vector(-2.0,0,-1.4), 0.2, 1.0,c)
																										     )
																								       )
																							 )
																					   )
																			     //new BlobVertex(Vector(-1.4,0,-1.5), 0.4, 1.0,c)
																			     )
																	       )
																 )
														   )//bras 2
												     )
										       )
									 )


							   ),
					     new BlobBlend(
							   new BlobVertex(Vector(0.3,0.6,0.3), 0.5, 1.0,b),//oeil1
							   new BlobVertex(Vector(-0.3,0.6,0.3), 0.5, 1.0,b)//oeil2
							   )

					     ),
			       new BlobUnion(
					     new BlobUnion(
							   new BlobBlend(
									 new BlobCone(Vector(0,0,0.6), Vector(0,0,1.5), 0.5, 0.0, 0.5, 1.0,b),
									 new BlobBlend(
										       new BlobVertex(Vector(0,0.5,1.6), 0.6, 0.80,b),
										       new BlobVertex(Vector(0,0.2,1.6), 0.7, 1.0,b)
										       )
									 ),
							   new BlobVertex(Vector(0,-0.1,0.8), 1.0, 0.6,b)
							   ),//chapeau
					     new BlobUnion(
							   new BlobVertex(Vector(0.0,-1.0,-2.5), 0.7, 1.0,c),
							   new BlobBlend(
									 new BlobSub(
										     new BlobVertex(Vector(0.0,0.1,-2.2), 1.95, 1.0,b),
										     new BlobBox(Vector(-2,-2,-1.8),Vector(2,2.0,0),0.4,1.0,b)
										     ),
									 new BlobBlend(
										       new BlobBlend(
												     new BlobVertex(Vector(-0.4,0.0,-3), 0.5, 1.0,b),
												     new BlobBlend(
														   new BlobVertex(Vector(-0.4,0.0,-3.2), 0.5, 1.0,b),
														   new BlobBlend(
																 new BlobVertex(Vector(-0.5,0.0,-3.4), 0.5, 1.0,b),
																 new BlobSub(
																	     new BlobBlend(
																			   new BlobVertex(Vector(-0.6,0.0,-3.8), 0.8, 1.0,b),
																			   new BlobBlend(
																					 new BlobVertex(Vector(-0.6,0.3,-3.8), 0.8, 1.0,b),
																					 new BlobVertex(Vector(-0.6,0.5,-3.7), 0.5, 1.0,b)
																					 )
																			   ),
																	     new BlobBox(Vector(-2,-2,-5.8),Vector(2,2,-4),0.5,1.0,b)
																	     )//chaussure
																 )
														   )
												     ),
										       new BlobBlend(
												     new BlobVertex(Vector(0.4,0.0,-3), 0.5, 1.0,b),
												     new BlobBlend(
														   new BlobVertex(Vector(0.4,0.0,-3.2), 0.5, 1.0,b),
														   new BlobBlend(
																 new BlobVertex(Vector(0.5,0.0,-3.4), 0.5, 1.0,b),
																 new BlobSub(
																	     new BlobBlend(
																			   new BlobVertex(Vector(0.6,0.0,-3.8), 0.8, 1.0,b),
																			   new BlobBlend(
																					 new BlobVertex(Vector(0.6,0.3,-3.8), 0.8, 1.0,b),
																					 new BlobVertex(Vector(0.6,0.5,-3.7), 0.5, 1.0,b)
																					 )
																			   ),
																	     new BlobBox(Vector(-2,-2,-5.8),Vector(2,2,-4),0.5,1.0,b)
																	     )//chaussure
																 )
														   )
												     )
										       )
									 )
							   )//pantalon
					     )
			       );

  blob=new Blob(
		b1
		,0.5);
  Box box=blob->GetBox().Cubic();
  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);

}

void blobGround::init(){

  BlobNode* b1 =   new BlobBox(Vector(-100,-100,-5),Vector(100,100,0),1,1,Vector(0.13,0.8,0.1));
  int nb=80;
  std::vector< Vector > coords(nb);
  std::vector< double > intensitee(nb);
  std::vector< double > ray(nb);
  for(int i=0;i<nb;i++){
    double d = frand(10,20);
  loop:
    double x = frand(100,-100);
    double y = frand(100,-100);
    double z = 0;//frand(-4,-5);
    for(unsigned int j=0;j<coords.size();j++)
      if(Norm(Vector(coords[j][0],coords[j][1],coords[j][2])-Vector(x,y,z))<d)
	goto loop;//attention boucle infinie si mauvais parametres
    coords[i]= Vector(x,y,z);
    intensitee[i]=  frand(0.3,0.4);
    ray[i]= d;
  }

  for(int i=0;i<nb;i++){
    b1 = new BlobBlend(
		       b1,
		       new BlobVertex(coords[i], ray[i],intensitee[i],Vector(0.9,0.3,0.2))
		       );
  }
  blob=new Blob(
		b1
		,0.5);
  Box box=blob->GetBox().Cubic();
  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);
}


void blobPuit::init(){

  BlobNode* b1 =
    new BlobSub(
		new BlobCylinder(Vector(0,0,-5.0), Vector(0,0.0,2), 2, 1.0, 1.0,Vector(0.3,0.3,0.3)),
		new BlobBlend(
			      new BlobCylinder(Vector(0,0,1.2), Vector(0,0.0,2), 1, 1.0, 1.0,Vector(0.3,0.3,0.3)),
			      new BlobCylinder(Vector(0,0,1), Vector(0,0.0,2), 0.8, 1.0, 1.0,Vector(0.1,0.1,0.9))
			      )
		);

  Blob bTmp(b1,0.5);
  Box boxTmp=bTmp.GetBox().Cubic();
  BlobNode* b2 = bTmp.Erode(boxTmp,ERODE_V,1e-4,Vector(0.9,0.9,0.9), 0.2, 0.5, 0.2,0.4,Vector(0.0,0,0));
  blob=new Blob(
		new BlobSub( b1, b2)
		,0.5);
  Box box=blob->GetBox().Cubic();
  blob->Polygonize(box,POLYGONIZE_V,av,at,ac,avn,atn,1e-4);
}
