/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/
// Blob
// Changelog 05.01.24

// Blob include
#include "blobtore.h"
//#include "vector.h"
/*!
  \brief Create a point skeletal element.
  \param c Center.
  \param blend Blending function.
*/
BlobTore::BlobTore(const Vector &c, const Vector& n, const double& rayon,const double& r,const double& s,const Vector& color):center(c),n((n-c)/Norm(n-c)),rayon(rayon), blend(Blend(r,s))
{
  BlobTore::color=color;
  BlobTore::box=Box(c,rayon+r + 1.0);
}

/*!
  \brief Overloaded to minimize function calls.
*/
double BlobTore::Intensity(const Vector& p)
{  
  // Box 
  if (!box.Inside(p)) return 0.0; 
  /*
    cf: http://www.geometrictools.com/Documentation/Documentation.html

    Soit P, Q et K formant un triangle où Q est la projeté de P que le plan du cercle et K le projeté de Q sur le cercle. Soit C le centre du cercle et N l'orientation du plan du cercle, r le rayon.
    |P-K|*|P-K| = |P-Q|*|P-Q| + |Q-K|*|Q-K|
    =(N.(P-C))*(N.(P-C)) + (|Q-C|-r)*(|Q-C|-r)
    =(N.(P-C))*(N.(P-C)) + (sqrt(|P-C|*|P-C| - (N.(P-C))*(N.(P-C)))-r) * (sqrt(|P-C|*|P-C| - (N.(P-C))*(N.(P-C)))-r) 
  */
  Vector x = center-p;
  double y = x*n;//(N.(P-C))
  double z = sqrt(x*x - y*y) -rayon;//(sqrt(|P-C|*|P-C| - (N.(P-C))*(N.(P-C)))-r)
  double r = y*y + z * z ;

  return blend(r);
}


Vector BlobTore::Color(const Vector& p){
  return color;
}
