#ifndef CBBOX_h
#define CBBOX_h
#include "vector.h"
#include "matrice.h"
//
// Oriented Bounding Box
//
// Stored as a matrix( without scaling ) and Extents( x, y, z )
//
class CBBox
{
 public:
  CBBox() {}
  CBBox( const CMatrix &M, const Vector &Extent ) 
    { Set( M, Extent );	}
  // BL = Low values corner point, BH = High values corner point
  CBBox( const CMatrix &M, const Vector &BL, const Vector &BH ) 
    { Set( M, BL, BH );	}
	
  void Set( const CMatrix &M, const Vector &Extent )
  {
    m_M = M;
    m_Extent = Extent;
  }	
  void Set( const CMatrix &M, const Vector &BL, const Vector &BH )
  {
    m_M = M;
    m_M.Translate( (BH + BL) * 0.5f );
    m_Extent = (BH - BL) / 2.0f;
  }
	
  Vector GetSize() 
  { return m_Extent * 2.0f; }
  Vector GetCenterPoint() 
  { return m_M.GetTranslate(); }		
  void GetInvRot( Vector *pvRot );
	
  bool IsPointInBox( const Vector& P );
  bool IsBoxInBox( CBBox &BBox );
  bool IsSphereInBox( const Vector& P, float fRadius );
  bool IsLineInBox( const Vector& L1, const Vector& L2 );
  bool BoxOutsidePlane( const Vector &Norm, const Vector &P );
	
  // Data
  CMatrix m_M;
  Vector m_Extent;		
};


#endif
