/**
 *
 * Copyright (C) 2013 Olivier Berthier
 *                    http://www.xunil.fr
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 **/

// Affichage OpenGL
// Changelog 06.02.2002

#include <iostream>

using namespace std;
#include <GL/glut.h>
#include <GL/gl.h>										
#include <GL/glu.h>
#include <vector>
#include <list>


#include "opengl.h"
#include "scene.h"
//#include "blobobject.h"
// Identifiant d'un cube
GLuint cube;

// Identifiant d'un quad
GLuint cut;

scene scene;


// Parametre global de temps
float t=0.0f;


// Angle
float alpha=0.0f;
float beta=180.0f;
float delta=0.0f;
float theta=-90.0f;

float zoom=-20.0f;
int Mousekey=0,Mousex,Mousey;

int light=1;
int rotate=0; 
int field=0; 
int blobby=1; 

// Identifiant de fenetre
int window;

void SpecialKeyboard(int key, int x, int y){

  switch (key)
    {
    case GLUT_KEY_LEFT :
      beta+=0.95*2;
      break;
    case GLUT_KEY_RIGHT :
      beta-=0.95*2;
      break;
    case GLUT_KEY_UP :
      theta+=0.95*2;
      break;
    case GLUT_KEY_DOWN :
      theta-=0.95*2;
      break;
    }

}
// Gestion du clavier.
void Keyboard(unsigned char key, int x, int y)
{
  // Si ESC
  if (key==27)
    {
      glutDestroyWindow(window); 
      exit(0);
    }

  //      alpha+=0.95;

  // Gestion des lumieres
  if ((key=='l') || (key=='L')) {
      light = 1-light;
      if (!light) 
	  glDisable(GL_LIGHTING);
      else 
	  glEnable(GL_LIGHTING);
    }
  if ((key=='a') || (key=='A'))
    zoom++;
  if ((key=='s') || (key=='S'))
    zoom--;

  if ((key=='r') || (key=='R'))
      rotate = 1-rotate;
  if ((key=='b') || (key=='B'))
      blobby = 1-blobby;
  if ((key=='p') || (key=='P'))
      field = 1-field;
  if (key=='+'){
      scene.cutpos_move(1);
      scene.GenerateCut();
      // cut=scene.GenerateCut();
    }
  if (key=='-') {
      scene.cutpos_move(-1);
      scene.GenerateCut();
      // cut=scene.GenerateCut();
    }


  glutPostRedisplay();
}

void Resize(int width, int height)
{
  // Evite de divisser par 0 en cas de changement de taille
  if (height==0)				
    {
      height=1;
    }
  // Fenetrage
  glViewport (0, 0, width, height);

  // Definition de la matrice de vue
  glMatrixMode (GL_PROJECTION);
  glLoadIdentity ();  

  // Camera
  gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,1000.0f);

  // Rebascule dans le repere du modele
  glMatrixMode(GL_MODELVIEW);
}


// Affichage
void GlutRendering()
{
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

  glMatrixMode (GL_MODELVIEW);
  glLoadIdentity ();

  glTranslatef(0.0f,0.0f,zoom);	

  if (rotate)
    {
      alpha+=0.95;
    }
  glRotatef(alpha,1.0f,1.0f,1.0f);
  glRotatef(beta,0.0f,1.0f,0.0f);
  glRotatef(delta,0.0f,0.0f,1.0f);
  glRotatef(theta,1.0f,0.0f,0.0f);

  scene.Rendering(blobby,field);
  glutSwapBuffers();
}

void MouseMove(int x, int y)
{
  Mousex = x;
  Mousey = y;
  /*	if(x >= 0 && x <= 512 && y >= 0 && y <= 512)
	{
		theta-=0.95*((y-256)/256.0);
		beta+=0.95*((x-256)/256.0);
		}*/
}

void MousePush(int button, int state, int x, int y)
{

  Mousex = x;
  Mousey = y;
  Mousekey=button;
}

// Rafraichissement
void GlutIdle(void)
{
  // Augmentation du parametre de temps
  t+=1.0;
  GlutRendering();
}

//!Initialise OpenGL
void InitGlut(int width,int height)
{  
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_ALPHA | GLUT_DEPTH);

  // Fenetre initiale
  glutInitWindowSize(512, 512);  
  glutInitWindowPosition(0, 0);  
  window=glutCreateWindow("Blobs");

  glutDisplayFunc(GlutRendering);
  //glutFullScreen();

  // Rafraichissement
  glutIdleFunc(&GlutIdle);

  // Changement de taille
  glutReshapeFunc(Resize);

  // Clavier
  glutKeyboardFunc(Keyboard);
  glutSpecialFunc(SpecialKeyboard);

  // Souris
  glutMouseFunc(MousePush);
  glutMotionFunc(MouseMove);

  // Initialise les parametres de rendu
  glClearColor (0.2f, 0.3f, 0.4f, 1.0);

  glClearDepth(1.0);
  glDepthFunc(GL_LEQUAL);
  glEnable(GL_DEPTH_TEST);	
  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glShadeModel(GL_SMOOTH);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();				// Reset The Projection Matrix

  gluPerspective(45.0f,(GLfloat)width/(GLfloat)height,0.1f,100.0f);

  // Placement des lumieres dans l'espace du modele
  glMatrixMode(GL_MODELVIEW);

  GLfloat light_ambient[] = { 0.5f, 0.5f, 0.5f, 1.0f };
  GLfloat light_diffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
  GLfloat light_position[] = { 0.0,0.0, 10.0,0.0 };

  glLightfv (GL_LIGHT1, GL_AMBIENT, light_ambient);
  glLightfv (GL_LIGHT1, GL_DIFFUSE, light_diffuse);
  glLightfv (GL_LIGHT1, GL_POSITION, light_position);
  glEnable(GL_LIGHT1); 
  glEnable(GL_LIGHTING);

  // Elimination des facettes arriere
  //glCullFace(GL_BACK);
  //glEnable(GL_CULL_FACE);
  // blobObject a;
  clock_t tStart = clock();
  scene.init();
  scene.GenerateBlob();
  scene.GenerateCut();
  
  printf("Temps de génération : %.2fs\n", (double)(clock() - tStart)/CLOCKS_PER_SEC);
  // cube=scene.GenerateBlob();
  //  cut=scene.GenerateCut();

}




int main(int argc,char **argv)
{
  glutInit(&argc, argv);
  
  InitGlut(512,512);
  glutMainLoop();
  return 0;
}
